#!/usr/bin/perl

package output;

use strict;
require 'two_dimensional.pm';
require 'html.pm';

use vars qw( $data $config @allkeys );

*data = *main::data;
*config = *main::config;

print "Generating output with all data (complex) <output/all.pm>\n" if $main::debug;

sub dump
# for debug purposes only.
{
	my @allkeys = sort (keys %$data);
	my ($key1, $val1, $key2, $val2, $amount);
	foreach $key1 (@allkeys)
	{
		foreach $val1 (sort (keys %{$data->{$key1}}))
		{
			foreach $key2 (@allkeys)
			{
				foreach $val2 (sort (keys %{$data->{$key1}{$val1}{$key2}}))
				{
					$amount = $data->{$key1}{$val1}{$key2}{$val2};
					print join (':', ($key1, $val1, $key2, $val2, $amount)) . "\n";
				}
			}
		}
	}
}

sub generate
{
	@allkeys = sort (@data::cntkeys, @data::sumkeys, @data::bytkeys);
	my ($key1, $val1, $key2, $val2, $amount, $percent);
	my $ignore = $config->{'ignore_less'};
	my $directory = '';
	if ($config->{'directory'})
	{
		$directory = $config->{'directory'};
		$directory =~ s/\/$//;
		if (!-d $directory) { die "Directory \"$directory\" does not exist\n"; }
		$directory .= '/';
	}

	foreach $key1 (@allkeys)
	{
		open (HTMLFILE, "> $directory$key1.html") or die $!;
		print HTMLFILE html::head_all ($key1, @allkeys);
		if ($config->{'draw_graph'})
		{
			require 'graph/makegraph.pm';
			makegraph::create_graph_png ($key1, $data);
			print HTMLFILE qq(<img src="$key1.png" width="600" height="250" border="0">\n\n);
		}

		foreach $val1 (sort (keys %{$data->{$key1}}))
		{
			my $general = $data->{$key1}{$val1}{$key1}{$val1};
			my ($html_val1) = html::escape ($val1);
			print HTMLFILE "<a name=\"$html_val1\">\n";
			print HTMLFILE qq(<table border="0" cellspacing="1" width="70\%">\n);
			print HTMLFILE qq(<tr><th colspan="5">$html_val1</th></tr>\n);

			foreach $key2 (sort (@data::sumkeys, @data::bytkeys))
			{
				next unless defined $data->{$key1}{$val1}{$key2}{'General'};

				$val2 = $data->{$key1}{$val1}{$key2}{'General'};
				if (grep ((($key1 eq $_) or ($key2 eq $_)), @data::bytkeys))
				{
					if ($val2 > 1073741824) { $val2 = sprintf ("%.1f GB", ($val2 / 1073741824)); }
					elsif ($val2 > 1048576) { $val2 = sprintf ("%.1f MB", ($val2 /    1048576)); }
					elsif ($val2 >    1024) { $val2 = sprintf ("%u kB"  , ($val2 /       1024)); }
					else { $val2 .= ' Bytes'; }
				}

				print HTMLFILE qq(<tr><td class="subhdr">$key2</td><td>&nbsp;</td><td>);
				print HTMLFILE $val2;
				print HTMLFILE q(</td><td>&nbsp;</td><td>&nbsp;</td></tr>);
				print HTMLFILE "\n";
			}

			KEY_TWO: foreach $key2 (@data::cntkeys)
			{
				if (($key1 eq $key2) or ($key2 eq 'General'))
				{
					next KEY_TWO;
				}
				my @allval2s = sort { $data->{$key1}{$val1}{$key2}{$b} <=> $data->{$key1}{$val1}{$key2}{$a} } (keys %{$data->{$key1}{$val1}{$key2}});
				my (@not_shown, @html_lines) = ((), ());
				my ($counter, $tmp) = (0, '');
				VAL_TWO: while (@allval2s)
				{
					$val2 = shift (@allval2s);
					my ($html_val2) = html::escape ($val2);
					$amount = $data->{$key1}{$val1}{$key2}{$val2};
					$percent = (int (10000 * $amount / $general) / 100);
					if ($percent < $ignore)
					{
						push (@not_shown, $html_val2);
						next VAL_TWO;
					}
					++$counter;
					if ($counter > 1) { $tmp = '<tr>'; }
					$tmp .= qq(<td>$html_val2</td><td>$amount</td><td>$percent\%</td><td>);
					my $cur_width = (int ((($config->{'bar'}{'width'} - 1) * $amount) / $general) + 1);
if ($cur_width > $config->{'bar'}{'width'}) { $cur_width = $config->{'bar'}{'width'}; }
					if ($cur_width)
					{
						$tmp .= "<img src=\"$config->{'bar'}{'src0'}\" width=\"" . $cur_width . "\" height=\"$config->{'bar'}{'height'}\" alt=\"$percent\%\">";
					}
					if ($cur_width < $config->{'bar'}{'width'})
					{
						$tmp .= "<img src=\"$config->{'bar'}{'src1'}\" width=\"" . ($config->{'bar'}{'width'} - $cur_width) . "\" height=\"$config->{'bar'}{'height'}\" alt=\"\">";
					}
					$tmp .= "</td></tr>\n";
					push (@html_lines, $tmp);
				}
				if (@not_shown)
				{
					$amount = scalar @not_shown;
					$tmp = qq(<tr><td colspan="4">Not shown: );
					if ($amount <= 7) { $tmp .= join ('; ', @not_shown); }
					else { $tmp .= join ('; ', @not_shown[0..6]) . ' (and ' . ($amount - 7) . ' more..)'; }
					$tmp .= "</td></tr>\n";
					push (@html_lines, $tmp);
				}
				my $lines = scalar @html_lines;
				print HTMLFILE "<tr>\n<td " . ($lines > 1 ? qq(rowspan="$lines" ) : '') . qq(class="subhdr">$key2</td>\n) . join ('', @html_lines) . "\n";
			}
			print HTMLFILE "</table>\n[ <a href=\"#beginofpage\">top</a> ]<br>\n<br>\n\n";
		}
		print HTMLFILE html::foot ();
		close HTMLFILE;
	}
	open (HTMLFILE, '> ' . $directory . 'index.html');
	print HTMLFILE html::head_all ('General', @allkeys);
	print HTMLFILE "<table border=\"0\" cellspacing=\"1\" width=\"50\%\">\n";
	print HTMLFILE "<tr><th colspan=\"2\">Overview</th></tr>\n";
	print HTMLFILE qq(<tr><td class="subhdr">Cathegory</td><td class="subhdr">Entries</td></tr>\n);
	foreach (@allkeys)
	{
		print HTMLFILE "<tr><td><a href=\"$_.html\">$_</a></td><td>" . scalar (keys %{$data->{$_}}) . "</td></tr>\n";
	}
	print HTMLFILE "</table>\n";
	print HTMLFILE "<br><br>\n\n";
	if (%$::extra)
	{
		print HTMLFILE qq(<table border="0" cellspacing="1" width="50\%">\n);
		print HTMLFILE qq(<tr><th colspan="2">General</th></tr>\n);
		print HTMLFILE qq(<tr><td class="subhdr">Key</td><td class="subhdr">Value</td></tr>\n);
		my ($key, $val);
		foreach $key (sort (keys %$::extra))
		{
			$val = $::extra->{$key};
			print HTMLFILE "<tr><td>$key</td><td>$val</td></tr>\n";
		}
		print HTMLFILE "</table>\n<br><br>\n";
	}
	print HTMLFILE html::foot ();
	close HTMLFILE;
}

1;
