package input;

use strict;
use vars qw( %month_numbers %extra );
require Tool;

print "Using NCSA/ACL (apache combined log) format <parser/ncsa.pm>\n" if $main::debug;

$extra{'total'} = 0;
$extra{'days'} = {};
$extra{'codered'} = 0;
$extra{'nimda'} = 0;
$extra{'search_terms'} = {};

data::init (	Host	=>	'count',
		User	=>	'count',
		Date	=>	'count',
		Hour	=>	'count',
		Domain	=>	'count',
		TLD	=>	'count',
		File	=>	'count',
		Status	=>	'count',
		Bytes	=>	'byte',
		Requests=>	'sum',
		Browser	=>	'count',
		OS	=>	'count',
		Referer	=>	'count'
	);

sub parse
{
	my $line = shift or return undef;
	if ($line =~ /^(\S+)\s(\S+)\s(.+?)\s\[(.+?)\]\s\"(.+?)\"\s(\d+)\s(.+?)\s\"(.+?)\"\s\"(.+?)\"(?:\s\"(.+?)\")?$/)
	{
# Initialize the variables that we can get out of
# each line first..
		my ($host, $ident, $user, $date, $request, $status, $bytes, $referer, $browser, $cookie) = ($1, $2, $3, $4, $5, $6, $7, $8, $9, $10);

# And now initialize all the variables we will use
# to get more information out of each field..
		my ($tmp, $day, $month, $year, $hour, $minute, $second, $method, $file, $params);
		($date, $tmp) = split (/ /, $date);
		($day, $month, $tmp) = split (/\//, $date);
		($year, $hour, $minute, $second) = split (/\:/, $tmp);
		if ($request ne '-')
		{
			($method, $tmp) = split (/ /, $request);
			($file, $params) = split (/\?/, $tmp, 2);
			$params = '' unless defined $params;
		}
		else { ($method, $file, $params) = ('', '', ''); }

		my $attack = 0;
		if (lc ($file) eq '/default.ida') { ++$extra{'codered'}; $attack = 1; }
		elsif (lc ($file) eq '/scripts/root.exe') { ++$extra{'nimda'}; $attack = 1; }
		elsif ($file =~ /root\.exe|winnt/i) { $attack = 1; }
		if ($attack and !$::config->{'inc_attacks'}) { return 'attack'; }

		my $tld;
		my $domain;
		if ($user eq '-') { $user = ''; }
		elsif (($user ne '-') and ($status >= 400) and ($status < 500)) { $user = 'invalid'; }

		if ($host =~ /(?:.+\.)?(.+?)\.([a-z]{2,4})$/)
		{
			$tld = lc ($2);
			$domain = lc ($1) . ".$tld";
		}
		else
		{       
			$domain = 'UNRESOLVED';
			$tld = 'UNRESOLVED';
		}

		my $os = Tool::detect_os ($browser);
		my $browser_name = Tool::detect_browser ($browser);
		my $n_date = $year . '-' . $month_numbers{$month} . '-' . $day;
		if ($user eq '-') { $user = 'unknown'; }
		if ($bytes eq '-') { $bytes = 0; }
		my @search_terms = Tool::extract_data ($referer);
		$referer = Tool::detect_referer ($referer);
		if ($referer eq '-') { $referer = ''; }

		if (!defined $extra{'days'}->{$n_date}) { $extra{'days'}->{$n_date} = 1; }
		++$extra{'total'};

		if (scalar @search_terms)
		{
			print "## TERMS " . join (' ', @search_terms) . "\n" if $::debug > 1;
			my $term = '';
			foreach $term (@search_terms)
			{
				if (!defined $extra{'search_terms'}->{$term}) { $extra{'search_terms'}->{$term} = 1; }
				else { ++$extra{'search_terms'}->{$term}; }
			}
		}

		my %combined = (
					'Host'		=>	$host,
					'User'		=>	$user,
					'Date'		=>	$n_date,
					'Hour'		=>	$hour,
					'Browser'	=>	$browser_name,
					'OS'		=>	$os,
					'Domain'	=>	$domain,
					'TLD'		=>	$tld,
					'File'		=>	$file,
					'Referer'	=>	$referer,
					'Status'	=>	$status,
					'Bytes'		=>	$bytes,
					'Requests'	=>	1
				);
		data::save (\%combined);
	}
	elsif ($::debug) { chomp ($line); print "$line\n"; }
}

sub extra
{
	my ($average, $days) = (0, 0);
	$days = scalar keys %{$extra{'days'}};
	$average = int (($extra{'total'} * 10) / $days) / 10;
	$::extra->{'Total requests'} = $extra{'total'};
	$::extra->{'Averange requests per day'} = $average;
	$::extra->{'Reporting period'} = "$days days";
	$::extra->{'CodeRed attacks'} = $extra{'codered'};
	$::extra->{'Nimda attacks'} = $extra{'nimda'};
	my (@tmp, $key, $val);
	foreach $key (sort { $extra{'search_terms'}->{$b} <=> $extra{'search_terms'}->{$a} } (keys %{$extra{'search_terms'}}))
	{
		$val = $extra{'search_terms'}->{$key};
		push (@tmp, "$key ($val)");
	}
	$::extra->{'Search terms used'} = join (', ', @tmp);
}

1;
