package input;

use strict;
use vars qw( %month_numbers %extra %names );

print "Using NET-ACCT format <parser/netacct.pm>\n" if $main::debug;

open (OWNCONFIG, '< netacct.config') or die $!;
while (my $line = <OWNCONFIG>)
{
	chomp ($line);
	$line =~ s/^(.*?)#.+$/$1/;
	
	next unless $line;
	my ($ip, $name) = split (/[\s\t]+/, $line) or next;
	if ($ip !~ /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/ or !$name) { next; }

	$names{$ip} = $name;
}
close (OWNCONFIG);

data::init (	Protocol	=>	'count',
		Source		=>	'count',
		SourcePort	=>	'count',
		Destination	=>	'count',
		DestinationPort	=>	'count',
		PacketCount	=>	'sum',
		Interface	=>	'count',
		User		=>	'count',
		Bytes		=>	'byte',
		Hour		=>	'count',
		Date		=>	'count',
		Month		=>	'count',
		Connections	=>	'sum'		);

sub parse
{
	my $line = shift or return undef;

	my @data = split (/[\t\s]+/, $line, 10);

# Initialize the variables that we can get out of
# each line first..
		my ($epoch, $protocol, $source_ip, $source_port, $dest_ip, $dest_port, $packet_count, $data_size, $interface, $user) = @data;

		my ($hour, $day, $month, $year) = (localtime ($epoch))[2,3,4,5];
		++$month; $year += 1900;
		my $date = sprintf ("%04u-%02u-%02u", $year, $month, $day);
		$hour = sprintf ("%02u", $hour);
		$month = sprintf ("%02u", $month);

# And now initialize all the variables we will use
# to get more information out of each field..

		if ($protocol == 1) { $protocol = 'ICMP'; }
		elsif ($protocol == 6) { $protocol = 'TCP'; }
		elsif ($protocol == 17) { $protocol = 'UDP'; }

		if (defined $names{$source_ip}) { $source_ip = $names{$source_ip}; }
		elsif ($source_ip eq '127.0.0.1') { $source_ip = 'localhost'; }
		elsif ($source_ip =~ /^192\.168\./) { $source_ip = 'lan'; }
		else { $source_ip = 'extern'; }
		
		if (defined $names{$dest_ip}) { $dest_ip = $names{$dest_ip}; }
		elsif ($dest_ip eq '127.0.0.1') { $dest_ip = 'localhost'; }
		elsif ($dest_ip =~ /^192\.168\./) { $dest_ip = 'lan'; }
		else { $dest_ip = 'extern'; }

		my %combined = (
					'Protocol'	=>	$protocol,
					'Source'	=>	$source_ip,
					'SourcePort'	=>	$source_port,
					'Destination'	=>	$dest_ip,
					'DestinationPort'=>	$dest_port,
					'PacketCount'	=>	$packet_count,
					'Interface'	=>	$interface,
					'User'		=>	$user,
					'Bytes'		=>	$data_size,
					'Hour'		=>	$hour,
					'Date'		=>	$date,
					'Month'		=>	$month,
					'Connections'	=>	1
										);
		data::save (\%combined);
}

sub extra
{
	# foo
}

1;
